/*
  
  Alloc/free/print LP_PROBLEMs and LP_SOLUTIONs
  
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "lp.h"
#include "xmalloc.h"
#include "minit.h"

struct LP_PROBLEM *
new_lp_problem (int var, int ineq, int eq)
{
  struct LP_PROBLEM *lp = xmalloc (sizeof (struct LP_PROBLEM));

  int m = ineq + eq;
  int i;
  lp->no_vars = var;
  lp->no_constraints = m;
  lp->no_eq_constraints = eq;
  
  lp->A = xcalloc(m,sizeof (Real*));


  for(i=0;i<m;i++)
    lp->A[i] =  xcalloc(var, sizeof(Real));

  lp->b =  xcalloc(m,sizeof(Real));
  lp->c =  xcalloc(var,sizeof(Real));

  return lp;
}

void
del_lp_problem (struct  LP_PROBLEM *lp)
{
  int i;
  free (lp->b);
  free (lp->c);

  for (i =0; i < lp->no_constraints; i++)
    {
      free (lp->A[i]);
    }
  free (lp->A);
  free (lp);
}

struct LP_SOLUTION *
new_lp_solution(int n, int m)
{
  struct LP_SOLUTION *sol = xmalloc (sizeof (struct LP_SOLUTION));

  sol->n = n;
  sol->m = m;
  sol->prim = xcalloc (n, sizeof (Real));
  sol->dual = xcalloc (m, sizeof (Real));
  sol->val = 0;
  return sol;
}

void
del_lp_solution (struct LP_SOLUTION*sol)
{
  free (sol->prim);
  free (sol->dual);
  free (sol);
}
void
init_solution (struct LP_SOLUTION *s)
{
  s->val = 0;
  memset (s->prim, 0, sizeof (Real) * s->n);
  memset (s->dual, 0, sizeof (Real) * s->m);  
}

struct LP_PROBLEM *
read_problem (FILE *in, int suppress)
{
  int n,m1,m2,m,i,j;
  int read = 0;
  int scanned = 0;
  struct LP_PROBLEM *lpp;
  Real **A, *b,*c;

int  expect;
 
  if (!suppress)
    fputs("Enter value of n (No. of unknowns): ", stderr);

  scanned = fscanf(in,"%d",&n);
  if (scanned > 0) read += scanned;
  if (!suppress)
    fputs("Enter no. of inequality constraints: ", stderr);

  scanned =  fscanf(in,"%d",&m1);
  if (scanned > 0) read += scanned;
  if (!suppress)
    fputs("Enter no. of equality constraints: ", stderr);

  scanned =  fscanf(in,"%d",&m2);
  if (scanned > 0) read += scanned;
  if (m1 < 0 || m2 <0 || n < 0)
    {
      fprintf (stderr, "need non-negative number of vars/constraints\n");
      return 0;
    }
  
  m = m1+m2;

  /* Allocate space for the arrays */
  lpp = new_lp_problem (n, m1, m2);
  A = lpp->A;
  b = lpp->b;
  c = lpp->c;
  
  if (!suppress)
    fprintf(stderr,"Enter (%d by %d) elements of A:\n", m,n);
  
  for(i=0;i<m;i++)
    for(j=0;j<n;j++)
      {
	scanned =  fscanf(in,REAL_FORMAT,&	A[i][j]);
	if (scanned > 0) read += scanned;
      }

  if (!suppress)
    fprintf(stderr,
	    "Enter %d elements of b, entered as a linear array:\n ",
	    m);
  for(i=0;i<m;i++)
    {
      scanned =  fscanf(in,REAL_FORMAT,&b[i]);
      if (scanned > 0) read += scanned;
    }

  if (!suppress)
    fprintf(stderr,
	    "Enter %d elements of c, entered as a linear array:\n",
	    n);
  for(j=0;j<n;j++)
    {
      scanned =  fscanf(in, REAL_FORMAT,&c[j]);
      if (scanned > 0) read += scanned;  }

  expect= m + n + m*n + 3;
  if (read != expect)
    {
      fprintf (stderr, "invalid format: read %d reals, expected %d.\n",  read, expect);
      return 0;
    }
  
  if (suppress)
    {
      print_problem_size (lpp);

    }
  return lpp;
}

void write_lp_problem_to_file (struct LP_PROBLEM const *p, char *filename)
{
  FILE *out = fopen (filename, "w");
  if (!out)
    {
      fprintf (stderr, "Can't open `%s\'\n as output", filename);
      return;
    }
  write_lp_problem (p, out);
  fclose (out);
}

void write_lp_problem (struct LP_PROBLEM const *p, FILE*out)
{
  int m = p->no_constraints;
  int i,j;

  char *format = "%10.5f ";
  
  fprintf(out,"%d %d %d\n\n",p->no_vars,
	  m - p->no_eq_constraints,
	  p->no_eq_constraints);
    
  for(i=0;i< m; i++)
      {
	for(j=0;j< p->no_vars; j++)
	  
	  fprintf(out,format, p->A[i][j]);
	fprintf(out,"\n");
      }
  
  fprintf (out, "\n\n");

  for(i=0;i<m;i++)
    {
      fprintf (out, format, p->b[i]);
    }

  fprintf (out, "\n\n");
  for(j=0;j<p->no_vars;j++)
    {
      fprintf(out, format, p->c[j]);
    }
  fprintf(out,"\n");
}


void
print_problem_size (struct LP_PROBLEM const *lpp)
{
  fputs("INPUT DATA:\n",stderr);
  fprintf(stderr,"No. of variables (n) = %d\n", lpp->no_vars);
  fprintf(stderr, "No. of inequality constraints (m1) = %d\n", lpp->no_constraints - lpp->no_eq_constraints);
  fprintf(stderr, "No. of equality constraints (m2) = %d\n", lpp->no_eq_constraints);
}

void
print_solution (struct LP_SOLUTION const *lps, Real precision)
{
  char format[64];
  int i,j;
  sprintf(format,"\nValue of z: \t%%%gg\n",precision);
  printf(format,lps->val);
  puts("Primal solution:");

  sprintf(format,"\t%%%gg ",precision);
  for(j=0;j< lps->n;j++)
    printf(format,lps->prim[j]);

  puts("\nDual solution:");
  for(i=0;i< lps->m;i++)
    printf(format,lps->dual[i]);

  putchar('\n');

}

