/*
  This is a front end to test the linear programming package
  minit. This algorithm was written by Das and Salazar in algol.
  
  Solve the LP
  
  		max C^T X  subj. to
                AX <= B
                 X >= 0

*/

/*

  hacked to bits by HWN.

*/

#include <getopt.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "minit.h"
#include "lp.h"

void
help ()
{
  fprintf (stderr,
	   "Usage:\tminit [options] [filenames]\n"
	   "Options:\n"
	   "  -pPREC  precision for output\n"
	   "  -d      debug\n"
	   "  -h      this help\n"
	   );
  
}

Real precision;
int suppress=1;

void
do_file (char *s)
{
  FILE *in =  (s) ? fopen (s, "r") :  stdin;
  struct LP_PROBLEM *lpp;
  struct LP_SOLUTION *lps;
  int err;
  if (!in)
    {
    fprintf (stderr, "Can't open `%s\'\n", s);
    return; 
    }
  
  lpp= read_problem (in, suppress);
  if (!lpp)
    return;
    
  lps = new_lp_solution (lpp->no_vars, lpp->no_constraints);
 err = lp_solve(lpp,lps);

  if (err)
    {
      fputs (lp_solve_error (err), stderr);
      exit (err);
    }
  if (s)
    fclose (in);


  print_solution (lps, precision);

  if (debug_minit)
    write_lp_problem (lpp, stdout);
}


int
main(int argc, char **argv)
{
  char c;
  precision = 0.0;

  while ((c = getopt (argc, argv, "hp:d")) != EOF)
    {
      int s;
      int int_part, frac_part;
      
      switch (c)
	{
	case 'd':
	  suppress = 0;
	  debug_minit = 1;
	  break;
	  
	case 'h':
	  help ();
	  exit (0);
	  break;
	case 'p':
	  s = sscanf(optarg, REAL_FORMAT,&precision);
	  /* Make sure precision is acceptible. */
	  int_part = precision;
	  frac_part = precision - int_part;
	  
	  if (int_part <= frac_part || int_part > 25 || s != 1)
	    {
	      fprintf (stderr,"Bad precision %g\n", precision);
	      precision = 10.3;	/* The default precision. */
	    }

	  break;
	}
    }

  if (optind >= argc)
    {
      do_file (NULL);
    }
  else
    while (optind < argc)
      {
	do_file (argv[optind++]);
      }
       
  
  exit(0);
}
